@@ -1,5 +1,6 @@
 import asyncio
 import inspect
+from collections import defaultdict
 from logging import getLogger
 from random import randint
 from time import time, time_ns
@@ -29,8 +30,11 @@ class Connection(BaseConnection):
         'time_delta',
         'subscriptions',
         'address',
+        '_sub_id',
         '_stream',
         '_listener',
+        '_sender',
+        '_pinger',
         '_recv_pool',
     )
     PROTOCOL_VERSION: int = 2
@@ -40,8 +44,11 @@ class Connection(BaseConnection):
         super().__init__(conf)
         self.api_version: int = api_version
         self.time_delta: float = 0.0
-        self.subscriptions: dict[int, Callable[[Action], Awaitable[None] | None]] = {}
+        self.subscriptions: dict[int, dict[int, Callable[[Action], Awaitable[None] | None]]] = defaultdict(dict)
+        self._sub_id: int = 0
         self._listener: asyncio.Task | None = None
+        self._sender: asyncio.Task | None = None
+        self._pinger: asyncio.Task | None = None
         self._recv_pool: dict[int, asyncio.Future] = {}
         self._stream: IOStream | None = None
         self.address: tuple[str, int] = '0.0.0.0', 0
@@ -56,8 +63,13 @@ class Connection(BaseConnection):
 
     async def start(self) -> None:
         await self.init()
-        self._loop.create_task(self.send_loop()).add_done_callback(self.on_tick_done)
-        self._loop.create_task(self.ping()).add_done_callback(self.on_tick_done)
+
+        self._sender = self._loop.create_task(self.send_loop())
+        self._pinger = self._loop.create_task(self.ping())
+
+        self._sender.add_done_callback(self.on_tick_done)
+        self._pinger.add_done_callback(self.on_tick_done)
+
         await self.recv_loop()
 
     async def init(self):
@@ -68,17 +80,17 @@ class Connection(BaseConnection):
 
         client_stmt = ClientStatement(
             api=self.api_version,
-            clientTime=time_ns() // 1000_000,
-            schemeFormat='JSON',
+            client_time=time_ns() // 1000_000,
+            scheme_format='JSON',
             compressors=['gzip', 'zlib'],
-            defaultCompression='zlib',
+            default_compression='zlib',
         )
         self.set_compressors(['gzip', 'zlib'], 'zlib')
         await self.write(client_stmt.pack())
         stmt: ServerStatement = ServerStatement.unpack(
             await self.read(as_uint(await self.read(4)))
         )
-        self.time_delta = (stmt.serverTime / 1000) - time()
+        self.time_delta = (stmt.server_time / 1000) - time()
 
         if self.conf.handshake is not None:
             await self.conf.handshake.send(self)
@@ -105,21 +117,29 @@ class Connection(BaseConnection):
 
     async def handle_broadcast(self, action: Action):
         if action.handler_id in self.subscriptions:
-            fn = self.subscriptions[action.handler_id]
-            res = fn(action)
-            if inspect.isawaitable(res):
-                await res
+            for fn in self.subscriptions[action.handler_id].values():
+                res = fn(action)
+                if inspect.isawaitable(res):
+                    await res
 
     async def handle_ping_action(self, action: PingAction):
         self.debug(f'Pong {action.send_time} [-] {action.recv_time}')
         await action.dump_data(0)
 
-    def subscribe(self, handler_id: int, handler: Callable[[Action], Awaitable[None] | None]):
-        self.subscriptions[handler_id] = handler
+    def subscribe(self, handler_id: int, handler: Callable[[Action], Awaitable[None] | None]) -> int:
+        self._sub_id += 1
+        self.subscriptions[handler_id][self._sub_id] = handler
+        return self._sub_id
 
-    def unsubscribe(self, handler_id: int):
+    def unsubscribe(self, handler_id: int, handler: Callable[[Action], Awaitable[None] | None] | int) -> None:
         if handler_id in self.subscriptions:
-            self.subscriptions.pop(handler_id)
+            if isinstance(handler, int):
+                self.subscriptions[handler_id].pop(handler, None)
+            else:
+                self.subscriptions[handler_id] = {
+                    k: v for k, v in self.subscriptions[handler_id]
+                    if v is not handler
+                }
 
     async def set_download_speed(self, speed=0):
         await self.write(b'\x05')
@@ -172,7 +192,7 @@ class Connection(BaseConnection):
     async def ping(self):
         if not self.conf.idle_timeout:
             return
-        wait = max(0.1, round(self.conf.idle_timeout / 2, 2))
+        wait = max(0.1, round(self.conf.idle_timeout * 0.9, 2))
         pong = PingAction()
         while self.is_open:
             await asyncio.sleep(wait)
@@ -190,3 +210,5 @@ class Connection(BaseConnection):
     def _close_tasks(self):
         yield from super()._close_tasks()
         yield from self._recv_pool.values()
+        yield self._pinger
+        yield self._sender
